function [realclass,x] = add_rotated_copies(realclass, x, pars)
    if ~isfield(pars,'bootstrap'), pars.bootstrap=3; end;
    if ~isfield(pars,'MAX_ANGLE'), pars.MAX_ANGLE=5; end;
    if ~isfield(pars,'ncPLOT'), pars.ncPLOT=0; end;
    if ~isfield(pars,'randnm'), pars.randnm=0; end;
    if ~isfield(pars,'idim'), pars.idim=30; end;
    
    if pars.randnm == 1
        ang = pars.MAX_ANGLE/3;
    else
        ang = pars.MAX_ANGLE;
    end
    
    %%select random pattern
    idx = floor(size(x,1)*rand([pars.ncopies 1]))+1;
        
    for i=1:length(idx)
        p(1,:) = x(idx(i),:);
        
        %create copy of pattern
        if pars.bootstrap==2 %only rotation
            p_copy = rotating(p,ang,pars.randnm,pars.idim);
        elseif pars.bootstrap==3 %only shortening
            p_copy = shortening(p,3);
        else
            p_copy = shortening(p,3);
            p_copy = rotating(p_copy,ang,pars.randnm,pars.idim);
        end
        
        %plot to testing
        if pars.ncPLOT == 1
            plotncopies(p,p_copy);
        end        
        
        %add the unchanged rest
        p_copy(1,91:92)=p(1,91:92);
        
        %add pattern to datasets
        x = [x; p_copy];
        realclass = [realclass realclass(idx(i))];

    end
end

function p_copy = shortening(p,n)

    rfirst = floor(rand * (n-1))+1;%cut the rfirst first sample
    rlast = floor(rand * (n-1))+1;%cut the rlast last sample
    idim = 30;
    
    k=1+rfirst:idim-rlast;
    psx = p(1,(0*idim)+k);
    psy = p(1,(1*idim)+k);
    psz = p(1,(2*idim)+k);

    t1 = [1:((idim-1)/(size(psx,2)-1)):idim];
    t2 = [1:1:idim];
    px_copy = interp1(t1,psx,t2);
    py_copy = interp1(t1,psy,t2);
    pz_copy = interp1(t1,psz,t2);

    p_copy = [px_copy py_copy pz_copy];

end

function p_copy = rotating(p,ang,randnm,idim)

    if randnm == 1
        %use normally distributed pseudorandom numbers between -3 and 3
        %to define a MAX_ANGLE (if necessary)
        random=randn(1,1);
        while (random>3 || random<-3)%max.3*ang% drehung
            random=randn(1,1);
        end
        alpha=(random*ang); 

        random=randn(1,1);
        while (random>3 || random<-3)%max.3*ang% drehung
            random=randn(1,1);
        end
        beta = (random*ang);

        random=randn(1,1);
        while (random>3 || random<-3)%max.3*ang% drehung
            random=randn(1,1);
        end
        gamma = (random*ang);
    else %random numbers (0,1)
        alpha=(((rand*2)-1)*ang);
        beta = (((rand*2)-1)*ang);
        gamma = (((rand*2)-1)*ang);
    end
    %fprintf('i=%d ,[alpha, beta, gamma] = [%8.3f,%8.3f,%8.3f]\n',i,alpha,beta,gamma);

    al0 = alpha*pi/180;             % BUG FIX /WK/
    be0 = beta*pi/180;
    ga0 = gamma*pi/180;

    rx = [1 0 0; 0 cos(al0) sin(al0); 0 -sin(al0) cos(al0)];
    ry = [cos(be0) 0 -sin(be0); 0 1 0; sin(be0) 0 cos(be0)];
    rz = [cos(ga0) sin(ga0) 0; -sin(ga0) cos(ga0) 0; 0 0 1];

    rm = rx*ry*rz;          %other combinations possible

    for k=1:idim
        v(1,:) = [p(1,(0*idim)+k) p(1,(1*idim)+k) p(1,(2*idim)+k)];

        v_rot(1,1:3) = v*rm;
        p_copy(1,[(0*idim)+k (1*idim)+k (2*idim)+k])=v_rot;
    end

end

function plotncopies(p,p_copy)
    idim = size(p_copy,2)/3;
    x_orig = p(1,1:idim);
    y_orig = p(1,idim+1:(2*idim));
    z_orig = p(1,(2*idim)+1:(3*idim));

    x_copy = p_copy(1,1:1:idim);
    y_copy = p_copy(1,idim+1:(2*idim));
    z_copy = p_copy(1,(2*idim)+1:(3*idim));


    figure(5); clf; set(gcf, 'Position', [89 77 866 638]);    
    subplot(2,1,1); hold on;
    plot(1:numel(x_orig), x_orig', '-b');
    plot(1:numel(y_orig), y_orig', '-r');
    plot(1:numel(z_orig), z_orig', '-g');
    plot(1:numel(x_copy), x_copy', '-.b');
    plot(1:numel(y_copy), y_copy', '-.r');
    plot(1:numel(z_copy), z_copy', '-.g');
    title('Original');
    xlabel('time');

    subplot(2,1,2); hold on;
    plot(1:numel(x_copy), x_copy', '-b');
    plot(1:numel(y_copy), y_copy', '-r');
    plot(1:numel(z_copy), z_copy', '-g');
    ti = 'Copy';%strcat('rotated alpha:', num2str(alpha),' beta:', num2str(beta),' gamma:', num2str(gamma));
    title(ti);
    xlabel('time');
    getframe(gcf);

end