function res = class_demo2B(new_xtst,new_realc_tst)
% class_demo2B: demonstrate how a stored SFA classification model can be 
%               loaded and used for prediction on new test data 
%
% class_demo2A + class_demo_2B are the same as class_demo2, 
% class_demo2A does only the training part, class_demo2B only the test
% part. The difference to class_demo2 is that all trained model
% elements are stored on file(s) and thus training and prediction can be invoked
% independently.
%
% Instead of handwritten digits we use the Vowel or WDBC benchmark dataset 
% from UCI machine learning repository (http://archive.ics.uci.edu/ml) 
% or gesture data (Hein/Bertram/Polak, FH Kln) from Cologne University, see
% dataLoad.m
%
% INPUT
%   new_xtst        (optional) NTST x IDIM, new input data (test set)
%   new_realc_tst   (optional) 1 x NTST, class labels for new_xtst
% Other data are reloaded from files:
%       data/Classif_class_demo2A.mat: the data {x, xtst, realclass, realc_tst, opts}
%       opts.sFilename:   the trained SFA model
%       opts.gFilename:   the trained Gauss classifier with SFA
%       opts.xFilename:   the trained Gauss classifier w/o SFA
% The first file is written by class_demo2A.m, the three others are
% written by sfaClassModel.m.
% If new_realc_tst is missing on input, then only a prediction is made, no
% error rate, no confusion matrix.
% If new_xtst is missing on input, then use {xtst,realc_tst} reloaded from
% data/Classif_class_demo2A.mat as substitute.
%
% OUTPUT
%   res         struct with the results from classify-run, e.g. 
%       errtst  1 x 2 matrix: error rate with / w/o SFA on test set 
%       ytst    output from SFA when applied to test data
%       predT   predictions with SFA + GaussClassifier on test set
%       predX   predictions w/o SFA (only GaussClassifier) on test set 
%
% Author: Wolfgang Konen, Jan 2011
%
% See also: sfaClassModel, dataLoad, CLASS_DEMO2, CLASS_DEMO2A 


    fprintf('Loading data {x, xtst, realclass, realc_tst, opts} from %s\n','data/Classif_class_demo2A.mat');
    load('data/Classif_class_demo2A.mat');  % 'x','xtst','realclass','realc_tst','opts' 
    
    if ~exist('new_xtst','var')
        new_xtst = xtst;
        new_realc_tst = realc_tst;
    end
    
    if exist('new_realc_tst','var')
        res = sfaClassPredict(new_xtst,opts,new_realc_tst);
        % confusion matrix & class error on test set
        [confmat,classerr] = mk_confmat(new_realc_tst,res.predT,opts.nclass);
        fprintf('Confusion matrix & class errors on test set\n    --- true class ---\n')
        disp(confmat);
        disp(num2str(classerr,'%7.3f'));        
    else 
        res = sfaClassPredict(new_xtst,opts);
    end

    
    

end 

