function [M,c_col,i_col,origin,info_preproc,opts] = dataLoadCV(opts,pars)
%
% dataLoadCV is a help routine for class_CVdemo2.m
%
    if pars.GENERATE_P==2
        % read Vowel data from
        % http://archive.ics.uci.edu/ml/datasets/Connectionist+Bench+%28Vow
        % el+Recognition+-+Deterding+Data%29  (UCI repository)
        M0=dlmread('data/vowel-context.data');   
        % IMPORTANT: let DLMREAD infer the delimeter (one or more
        % whitespaces) from file, because only in this way, consecutive
        % whitespaces are treated as ONE delimiter
        %
        M=M0(:,4:14);
        c_col = 11;         % class column
        i_col = 1:10;       % input columns range
    elseif pars.GENERATE_P==3
        % read Vowel data from
        % http://www.inf.ed.ac.uk/teaching/courses/inf2b/coursework/cwk3.html 
        % (Steve Renals Homework Assignment) 
        % Same IO-records as in GENERATE_P==2, but different divide in train & test
        %
        Mtrn=csvread('data/vowel-train.csv');
        Mtst=csvread('data/vowel-test.csv');
        M = [Mtrn; Mtst];
        c_col = 11;         % class column in M
        i_col = 1:10;       % input columns range in M
    elseif pars.GENERATE_P==4
        % http://archive.ics.uci.edu/ml/datasets/Breast+Cancer+Wisconsin+(Diagnostic)
        % UCI repository, breast cancer, with replacement in column 2:
        %   M >> -1 (malignant)   and     B >> 2 (benign)
        M=csvread('data/wdbc.data');   
        c_col = 2;          % class column in M
        i_col = 3:32;       % input columns range in M
        %M(:,i_col)=M(:,i_col)./repmat(mean(M(:,i_col)),size(M,1),1);
        %parallel_plot(M(:,i_col),M(:,c_col));
    elseif pars.GENERATE_P==9 
        % read gesture data 
        % as they were generated by beginner.m (Marco Polak, Kristine Hein)
        load('data/class_pat_gest.mat');
        M=[realclass' x];
        c_col = 1;          % class column in M 
        i_col = 2:31;       % input columns range in M
        opts.pp_range=11;
        %parallel_plot(x,realclass);              
    elseif pars.GENERATE_P==10
        % read gesture data 
        % as they were generated by load_wiigesturedata.m (Kristine Hein, WK)
        % (see <fiwa_soma>/data/Gesture.d/WiiGestureData_KH_I/):
        path='data/';
        %path='../../../data/Gesture.d/WiiGestureData_KH_I/';
        %file = 'hk_gesture_all-norm0.mat';      % all 716 patterns, w/o  amplitude normalization (NOT recommended)
        file = 'hk_gesture_all-norm1.mat';      % all 716 patterns, with amplitude normalization
        %file = 'hk_gesture_p130.mat';           % only person 130
        fname = [path file];
        if ~exist(fname,'file'), fname=['../demo/data/' file];, end;
        load(fname); 
        % File fname contains the variables info_preproc, realclass, x
        % and optionally also the variable origin.
        disp(sprintf('Read %7d   gesture patterns from %s',size(info_preproc,1),fname));
        if ~exist('origin','var'), origin='KH_I/setUnknown-norm*'; end
        [info_preproc,realclass,x] = select_subset_person(pars,info_preproc,realclass,x);
        M=[realclass' x];
        imax = size(x,2);       % 91 for -norm0, 92 for -norm1
        i_col = 2:(imax+1);     % input columns range in M
        c_col = 1;              % class column in M
        %opts.pp_range=11;
        title=sprintf('%d patterns (ID_{person}=%d)',numel(realclass),pars.PERSON_ID);
        %parallel_plot(x,realclass,16,title);     
    elseif pars.GENERATE_P==11
        % read gesture data 
        % as they were generated by load_wiigesturedata.m (Kristine Hein, WK)
        % (see <fiwa_soma>/data/Gesture.d/WiiGestureData_KH_II/):
        %path='data/';
        path='../../../data/Gesture.d/WiiGestureData_KH_II/';
        %file = 'hk_gesture_set1-norm0.mat';     % all 1730 patterns, w/o  amplitude normalization (NOT recommended)
        file = 'hk_gesture_set1-norm1.mat';      % all 1730 patterns, with amplitude normalization
        fname = [path file];
        if ~exist(fname,'file'), fname=['../demo/data/' file];, end;
        load(fname); 
        % File fname contains the variables info_preproc, realclass, x
        % and optionally also the variables gesture_meta and origin.
        disp(sprintf('Read %7d   gesture patterns from %s',size(info_preproc,1),fname));
        if ~exist('origin','var'), origin='KH_II/setUnknown-norm*'; end
        [info_preproc,realclass,x] = select_subset_person(pars,info_preproc,realclass,x);
        M=[realclass' x];
        %REDUCE_CLASS='KH_I';    % only gestures 1,...,5 which appear also in KH_I
         REDUCE_CLASS='iphone';  % only gestures 1,5,6,7,8,9 which appear also in iPhone set
        if (strcmp(REDUCE_CLASS,'KH_I'))
            disp(        '*** Reducing dataset to the gestures 1,...,5 ***');
            M=M(realclass<=5,:);
            info_preproc=info_preproc(realclass<=5,:);
            disp(sprintf('*** Remaining patterns:   %7d   ***',size(info_preproc,1)));
        end
        if (strcmp(REDUCE_CLASS,'iphone'))
            disp(        '*** Reducing dataset to the gestures 1,5,6,7,8,9 ***');
            M=M(realclass==1 | realclass>=5,:);
            info_preproc=info_preproc(realclass==1 | realclass>=5,:);
            disp(sprintf('*** Remaining patterns:   %7d   ***',size(info_preproc,1)));
        end
        imax = size(x,2);       % 91 for -norm0, 92 for -norm1
        i_col = 2:(imax+1);     % input columns range in M
        c_col = 1;              % class column in M
        %opts.pp_range=11;
        title=sprintf('%d patterns (ID_{person}=%d)',numel(realclass),pars.PERSON_ID);
        %parallel_plot(x,realclass,16,title);     
    elseif pars.GENERATE_P==12
        % read iphone gesture data 
        % as they were generated load_iphonegesturedata.m (Daniel Bertram, Kristine Hein)
        % (see <fiwa_soma>/data/Gesture.d/iPhoneGestureData/)
        path='data/';
        %path='../../../data/Gesture.d/iPhoneGestureData/';
        file = 'iphonegesture-norm1.mat';     % all 892 patterns, with amplitude normalization 
        fname = [path file];
        if ~exist(fname,'file'), fname=['../demo/data/' file];, end;
        load(fname)
        % File fname contains the variables info_preproc, realclass, x
        % and optionally also the variable origin.
        disp(sprintf('Read %7d   gesture patterns from %s',size(info_preproc,1),fname));
        if ~exist('origin','var'), origin='iPhone: normUnknown'; end
        [info_preproc,realclass,x] = select_subset_person(pars,info_preproc,realclass,x);
        M=[realclass' x];
        imax = size(x,2);       % 91 for -norm0, 92 for -norm1
        i_col = 2:imax+1;       % input columns range in M
        c_col = 1;              % class column in M
        %opts.pp_range=11;
        title=sprintf('%d patterns (ID_{person}=%d)',numel(realclass),pars.PERSON_ID);
        %parallel_plot(x,realclass,16,title);     
    end % pars.GENERATE_P
    
    opts.classes = unique(M(:,c_col));  % labels of the classes (we assume that each class appears
                                        % at least once in the training data)
    opts.nclass=numel(opts.classes);    % number of classes
    opts.gaussdim=opts.nclass-1;        % number of SFA-outputs used in Gauss classifier
    opts.idim=numel(i_col);             % number of input dimensions
    if ~isfield(opts,'pp_range')
        opts.pp_range=opts.idim;        % dims after preproc = number of input dims
    end
    if opts.pp_range>opts.idim
        opts.pp_range=opts.idim;        % dims after preproc may not be larger than number of input dims
    end
    %if pars.GENERATE_P==4
    %    opts.gaussdim=3;                % do not use only the slowest SFA output (because 
    %end                                 % nclass=2) but use the slowest *three* SFA outputs        
    if ~exist('origin','var'), origin='Unknown'; end
    if size(M,1)==0
        error('No data! (M is empty)');
    end 
end

function [info_preproc,realclass,x] = select_subset_person(pars,info_preproc,realclass,x)        
    if pars.PERSON_ID~=-1
        if (pars.PERSON_ID<-1)
            idx = find(info_preproc(:,1)<=abs(pars.PERSON_ID));
            disp(['Selected subset of data for all persons with PERSON_ID<=' num2str(abs(pars.PERSON_ID));]);
        else
            idx = find(info_preproc(:,1)==pars.PERSON_ID);
            disp(['Selected subset of data for PERSON_ID=' num2str(pars.PERSON_ID);]);
        end
        %idx = find(info_preproc(:,1)==pars.PERSON_ID & realclass'~=1);
        realclass = realclass(idx);
        x = x(idx,:);
        info_preproc = info_preproc(idx,:);
    end
end

