%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% helper function for class_*demo*.m:
% Divide data x into training set x and test set xtst.
% Ensure that each class appears at least once in the training set.
%
function [x,xtst,realclass,realc_tst] = divide_rand(x,realclass,pars)
    % divide x and realclass in test and training patterns by random split
    if pars.tstvarseed==0               
        if ~isfield(pars,'seed'), pars.seed=4; end
        rand('state',pars.seed);        % each time the same random numbers
    else
        rand('state',sum(100*clock));   % each time different random numbers
    end
    sz=size(x,1);
    tstind=zeros(1,sz);
    %--OLD VERSION, ONLY APPROX OK:
    %tstind(1,rand(1,sz)<pars.tstfrac) = 1;
    %--BETTER VERSION:
    tstind(1,1:(fix(sz*pars.tstfrac)+1)) = 1;
    tstind = tstind(randperm(sz));
    
    % 
    % the following ensures that each class appears at least once in the
    % training set:
    classes = unique(realclass);
    realc_trn = realclass(tstind==0);
    for i=1:numel(classes)
        cl = classes(i);
        if numel(find(realc_trn==cl))==0
            indcl=find(realclass==cl);
            indcl=randperm(indcl);      % a random index to the records with class=cl
            indtrn=find(tstind==0);
            indtrn=randperm(indtrn);    % a random index to the train records
            tstind(indcl(1))==0;        % swap a class-cl-record ...
            tstind(indtrn(1))==1;       % ... with a train record
        end
    end
    
    xtst = x(tstind==1,:);
    x    = x(tstind==0,:);
    realc_tst = realclass(tstind==1);
    realclass = realclass(tstind==0);
end    
