function [cpermat,cmatall,cpermatRel,classerr] = mk_cpermat(res)
% mk_cpermat: given the results from class_CVdemo2, calculate the error
% rates per person and per class. Make a bar plot of the per-person error rates.
%
% INPUT
%   res         output from running class_CVdemo2.m, e.g.
%                     res = class_CVdemo2()
%
% OUTPUT
%   cpermat     nperson x nclass matrix, cpermat(j,i): how often is true class i
%               predicted correctly for person j
%   cmatall     nperson x nclass matrix, cmatall(j,i): how many records with true 
%               class i are there for person j
%   cpermatRel  = cpermat ./ cmatall, the corresponding error rates per
%               person per class
%
% Author: Wolfgang Konen, Aug 2011
%
% See also: CLASS_CVDEMO2

    nclass=size(unique(res.realclass),1);
    classes=unique(res.realclass);
    nperson=size(unique(res.info_preproc(:,1)),1);
    persons=unique(res.info_preproc(:,1));
    cpermat = zeros(nperson,nclass);
    cmatall = zeros(nperson,nclass);
    for j=1:nperson  % row j: person j
        idx = find(res.info_preproc(:,1)==persons(j));
        nrec = numel(idx);
        for i=1:nclass  % column i: real class i
            CL=classes(i);
            cpermat(j,i)=numel(find(res.realclass(idx)==CL & res.cvpredict(idx)==CL));
            cmatall(j,i)=numel(find(res.realclass(idx)==CL));
        end
    end
    total = sum(cmatall);
    t_err = sum(cpermat);
    classerr = t_err./(total+1e-15);
    % why '+1e-15'? - regularize the case where a certain class does not
    % appear at all in the test set
    totalerr = sum(t_err)/sum(total);
    classerr = [classerr totalerr];
    cpermatRel = cpermat ./ cmatall;
    
    zz=[unique(res.info_preproc(:,1)) sum(cpermat,2)./sum(cmatall,2)];
    bar(zz(:,1),100*zz(:,2));ylim([50,100]);
    xlabel('person ID','FontSize',14); 
    ylabel('classification accuracy in %','FontSize',14);
    title(['Gestures 1-5 from dataset ', regexprep(res.pars.origin, '_', '\\_')],'FontSize',14);
end
  