function pars = parsDefault()
%   Set defaults for struct pars, a  struct with several parameter settings
%   for training and test data generation and for parametric bootstrap. 
%               
        pars.NRUN =10;       % execute the sweep NRUN times with different seeds to diminuish 
                             % statistical fluctuations in the error rates   
        pars.GENERATE_P=12;  % =1: load 'Vowel.csv' (R mlbench), divide randomly in tst/trn
                             % =2: load 'vowel-context.data' (UCI repository, recommended)  
                             % =3: load 'vowel-train.csv', 'vowel-test.csv' (Renal assignment)
                             % =4: load 'wdbc.data (UCI repository, malignant & benign breast cancer)
                             % =9: load 'class_pat_gest.mat' (gesture data Polak/Hein)
                             % =10: load 'hk_gesture_all-norm*.mat' (KH_I gesture data K.Hein) 
                             % =11: load 'hk_gesture_set*-norm*.mat' (KH_II gesture data K.Hein) 
                             % =12: load 'iphonegesture-norm*.mat' (iphone data D. Bertram, K.Hein) 
        pars.tstdiv=1;       % =1: divide randomly in tst/trn even in case GENERATE_P=2 or 3
                             % =0: keep tst/trn from file in case GENERATE_P=2 or =3
        pars.tstfrac=0.20;   % which fraction of the patterns become test 
                             % patterns in case of random divide
        pars.tstvarseed=1;   % =1: vary seed, =0: always seed 0, for random number generator
                             % in case of cross validation index 
        pars.nfold=10;       % number of folds in cross validation index
        pars.doPB=1;         % =1/0: do/don't automatic parametric bootstrap, if training set is too small
        pars.ncopies=  0;    % if >0, then class_CVdemo2.m will add unconditionally this many 'noisy copies'.
                             % If =0 and if pars.doPB=1, then sfaPBootstrap will conditionally add 
                             % parametric bootstrap patterns, if training set too small
        pars.ncalpha=1.5;    % If training set is too small, fill it with parametric bootstrap patterns to
                             % the desired size: pars.ncalpha*(D_xp + K)
        pars.ncsigma=1.0;    % the noise is pars.ncsigma*(standard deviation of input column),
                             % only relevant for parametric bootstrap
        pars.ncmethod=3;     % =1: "mu = pattern" in add_noisy_copies.m (OLD)
                             % =2: "mu = class centroid"  
                             % =3: "mu = class centroid" + "sigma = class stdev" (recommended, *real* parametric bootstrap)
        pars.bootstrap=1;    % =1: parametric bootstrap by adding noisy data
                             % =2: only rotation
                             % =3: only shorted copies
                             % =4: shorted + rotated copies
        pars.MAX_ANGLE = 10; % limit the allowed angles to [-MAX_ANGLE,MAX_ANGLE], for boostrap=2 & 4
        pars.PERSON_ID=  -1; % Only for pars.GENERATE_P==10,11,12:
                             % -1: all persons, +N: only records of a specific person, 
                             % -N: all records of persons <= |N|, 
                             % with N=121,...,130 for pars.GENERATE_P=10,11
                             % and N=1,2,4,5,..,15 for pars.GENERATE_P=12
        pars.CV_ON_PERSONS=0;% 0: create random CV index, 1: index=(person number-C)
        pars.n = 1;          % for write_gestureCV_csv   
end            
