function [realclass,x] = sfaPBootstrap(realclass,x,opts,pars)
% sfaPBootstrap: if training set too small, augment it with parametric bootstrap
%
% INPUT
%   x           NREC x IDIM, training input data
%   realclass   1 x NREC, training class labels
%   opts        struct with several parameter settings
%
% OUTPUT
%   x           training set extended to minimu number of records 
%               1.5*(xpdim+nclass), if necessary 
%   realclass   training class labels, extended analogously 
%   
% See also: ADD_NOISY_COPIES
    if opts.deg==1
        disp(sprintf('Warning: sfaPBootstrap not yet applicable for opts.deg==1 --> nothing done'));          
    else
        N_trig = ceil(1.2*(xp_dim(opts.pp_range)+opts.nclass));
        N_desi = ceil(pars.ncalpha*(xp_dim(opts.pp_range)+opts.nclass));
        N = size(x,1);
        if (N<=N_trig)
            disp(sprintf('Warning: # of training patterns = %d is less than the necessary 1.2*(D_x+K) = %d',N,N_trig));        
            if pars.doPB==0
                disp(sprintf('Warning: NOTHING done, because pars.doPB==0'));
            else 
                disp(sprintf('--> augmenting the training set with Parametric Bootstrap to %6.1f*(D_x+K) = %d',pars.ncalpha,N_desi));
                pars.ncopies=N_desi-N;
                [realclass, x] = add_noisy_copies(realclass,x,pars);
            end
        end
    end
end 

